@@ -21,7 +21,8 @@
 #include "../mediascanner/MediaStore.hh"
 #include "../mediascanner/MediaFile.hh"
 #include "InvalidationSender.hh"
-#include "MetadataExtractor.hh"
+#include "../extractor/DetectedFile.hh"
+#include "../extractor/MetadataExtractor.hh"
 #include "../mediascanner/internal/utils.hh"
 
 #include<sys/select.h>
@@ -135,6 +136,8 @@ void SubtreeWatcher::addDir(const string &root) {
         lstat(fullpath.c_str(), &statbuf);
         if(S_ISDIR(statbuf.st_mode)) {
             addDir(fullpath);
+        } else if (S_ISREG(statbuf.st_mode)) {
+            fileAdded(fullpath);
         }
     }
 }
@@ -219,10 +222,8 @@ void SubtreeWatcher::processEvents() {
         struct stat statbuf;
         lstat(abspath.c_str(), &statbuf);
         // Remember: these are not valid in case of delete event.
-        if(S_ISDIR(statbuf.st_mode))
-            is_dir = true;
-        if(S_ISREG(statbuf.st_mode))
-            is_file = true;
+        is_dir = S_ISDIR(statbuf.st_mode);
+        is_file = S_ISREG(statbuf.st_mode);
 
         if(event->mask & IN_CREATE) {
             if(is_dir) {
